from typing import List

class GenerateScheduling:
    """
    GenerateScheduling contains helper methods to generate scheduling arrays for use in persistent queries.
    """

    @staticmethod
    def _create_standard_parameters(scheduler_type: str,
                                   start_time: str,
                                   stop_time: str,
                                   time_zone: str,
                                   calendar: str,
                                   overnight: bool,
                                   repeat_enabled: bool,
                                   repeat_interval: int,
                                   skip_if_unsuccessful: bool,
                                   stop_time_disabled: bool,
                                   restart_error_count: int,
                                   restart_error_delay_minutes: int,
                                   restart_when_running: str):
        """
        Internal function to generate a base scheduler array with basic values. This array is not sufficient to create a valid
        scheduler by itself.

        :return: a base scheduling definition array that should be appended to create a valid scheduler.
        """
        if restart_when_running != "Yes" and restart_when_running != "No":
            raise ValueError("'restart_when_running' must be 'Yes' or 'No'")

        if repeat_enabled and repeat_interval < 1:
            raise ValueError("'repeat_interval' must be greater than zero when repeat is enabled")

        scheduling = []
        scheduling.append("SchedulerType=" + scheduler_type)
        if start_time is not None:
            scheduling.append("StartTime=" + start_time)
        if stop_time is not None:
            scheduling.append("StopTime=" + stop_time)
        scheduling.append("TimeZone=" + time_zone)
        if calendar is not None:
            scheduling.append("Calendar=" + calendar)
        scheduling.append("SkipIfUnsuccessful=" + str(skip_if_unsuccessful).lower())
        scheduling.append("StopTimeDisabled=" + str(stop_time_disabled).lower())
        scheduling.append("Overnight=" + str(overnight).lower())
        scheduling.append("RepeatEnabled=" + str(repeat_enabled).lower())
        if repeat_enabled:
            scheduling.append("RepeatInterval=" + str(repeat_interval))
        scheduling.append("RestartErrorCount=" + str(restart_error_count))
        scheduling.append("RestartErrorDelay=" + str(restart_error_delay_minutes))
        scheduling.append("RestartWhenRunning=" + restart_when_running)
        scheduling.append("SchedulingDisabled=false")

        return scheduling

    @staticmethod
    def generate_disabled_scheduler() -> List[str]:
        """
        Generates a scheduler array for a persistent query with scheduling disabled.

        :return: a scheduling definition array that can be passed to the controller when adding or updating a persistent query
        """
        scheduling = []
        scheduling.append(
            "SchedulerType=com.illumon.iris.controller.IrisQuerySchedulerDaily")
        scheduling.append(
            "Calendar=USNYSE")
        scheduling.append("BusinessDays=false")
        scheduling.append("Days=true=true=true=true=true=true=true")
        scheduling.append("StartTime=00:00:00")
        scheduling.append("StopTime=23:59:59")
        scheduling.append("TimeZone=America/New_York")
        scheduling.append("SchedulingDisabled=true")
        return scheduling

    @staticmethod
    def generate_daily_scheduler(start_time: str,
                                 stop_time: str,
                                 time_zone: str = "America/New_York",
                                 business_days: bool = False,
                                 calendar: str = "USNYSE",
                                 days: List[bool] = [True, True, True, True, True, True, True],
                                 overnight: bool = False,
                                 repeat_enabled: bool = False,
                                 repeat_interval: int = 0,
                                 skip_if_unsuccessful: bool = False,
                                 stop_time_disabled: bool = False,
                                 restart_error_count: int = 0,
                                 restart_error_delay_minutes: int = 0,
                                 restart_when_running: str = "Yes") -> List[str]:
        """
        Generate a scheduler array for a persistent query that has daily scheduling.

        :param start_time: the query's starting time in HH:MM:SS format
        :param stop_time: the query's stop time in HH:MM:SS format
        :param time_zone: the time zone, such as 'America/New_York' or 'Europe/London'
        :param business_days: if changed to True, only run on business days
        :param calendar: if business_days is True, the business calendar to use
        :param days: a list of bool values representing the days to run the persistent query, from Monday through Sunday
        :param overnight: if True, the query runs overnight
        :param repeat_enabled: for batch-style queries, if True, the persistent query will be restarted at the repeat_interval interval
        :param repeat_interval: if repeat_enabled is True, the interval in minutes between restarts
        :param skip_if_unsuccessful: for repeating queries, skip if the previous run failed when this is True
        :param stop_time_disabled: if True, the persistent query does not need a stop time (for example, non-repeating batch queries)
        :param restart_error_count: the number of times a failing query will attempt to be restarted within its run window
        :param restart_error_delay_minutes: the delay in minutes between attempting to restart a failed query
        :param restart_when_running: if 'Yes' then the query will be restarted when running at its start time, if 'No' it will be left running
        :return: a scheduling definition array that can be passed to the controller when adding or updating a persistent query
        """

        if len(days) != 7:
            raise ValueError("'days' must contain exactly 7 elements")

        scheduling = GenerateScheduling._create_standard_parameters("com.illumon.iris.controller.IrisQuerySchedulerDaily",
            start_time, stop_time, time_zone, calendar, overnight, repeat_enabled, repeat_interval, skip_if_unsuccessful,
            stop_time_disabled, restart_error_count, restart_error_delay_minutes, restart_when_running)
        scheduling.append("BusinessDays=" + str(business_days).lower())
        scheduling.append("Days=" + "=".join(str(item).lower() for item in days))

        return scheduling

    @staticmethod
    def generate_monthly_scheduler(start_time: str,
                                  stop_time: str,
                                  time_zone: str = "America/New_York",
                                  business_days: bool = False,
                                  calendar: str = "USNYSE",
                                  first_business_day: bool = False,
                                  last_business_day: bool = False,
                                  months: List[bool] = [True, True, True, True, True, True, True, True, True, True, True, True],
                                  days: List[int] = None,
                                  overnight: bool = False,
                                  repeat_enabled: bool = False,
                                  repeat_interval: int = 0,
                                  skip_if_unsuccessful: bool = False,
                                  stop_time_disabled: bool = False,
                                  restart_error_count: int = 0,
                                  restart_error_delay_minutes: int = 0,
                                  restart_when_running: str = "Yes") -> List[str]:
        """
        Generate a scheduler array for a persistent query that has monthly scheduling.

        :param start_time: the query's starting time in HH:MM:SS format
        :param stop_time: the query's stop time in HH:MM:SS format
        :param time_zone: the time zone, such as 'America/New_York' or 'Europe/London'
        :param business_days: if changed to True, only run on business days
        :param calendar: if business_days is True, the business calendar to use
        :param first_business_day: if True, then run the query on the first business day of the month
        :param last_business_day: if True, then run the query on the last business day of the month
        :param months: a List of the months on which the persistent query will be started, starting with January
        :param days: a list of specific days (from 1 to 31) to run the persistent query for each month, or None if this is not needed
        :param overnight: if True, the query runs overnight
        :param repeat_enabled: for batch-style queries, if True, the persistent query will be restarted at the repeat_interval interval
        :param repeat_interval: if repeat_enabled is True, the interval in minutes between restarts
        :param skip_if_unsuccessful: for repeating queries, skip if the previous run failed when this is True
        :param stop_time_disabled: if True, the persistent query does not need a stop time (for example, non-repeating batch queries)
        :param restart_error_count: the number of times a failing query will attempt to be restarted within its run window
        :param restart_error_delay_minutes: the delay in minutes between attempting to restart a failed query
        :param restart_when_running: if 'Yes' then the query will be restarted when running at its start time, if 'No' it will be left running
        :return: a scheduling definition array that can be passed to the controller when adding or updating a persistent query
        """

        if len(months) != 12:
            raise ValueError("'months' must contain exactly 12 elements")

        if days is not None and len(days) > 31:
            raise ValueError("'days' must contain 31 or fewer elements")

        scheduling = GenerateScheduling._create_standard_parameters("com.illumon.iris.controller.IrisQuerySchedulerMonthly",
            start_time, stop_time, time_zone, calendar, overnight, repeat_enabled, repeat_interval, skip_if_unsuccessful,
            stop_time_disabled, restart_error_count, restart_error_delay_minutes, restart_when_running)

        scheduling.append("BusinessDays=" + str(business_days).lower())
        scheduling.append("FirstBusinessDay=" + str(first_business_day).lower())
        scheduling.append("LastBusinessDay=" + str(last_business_day).lower())
        scheduling.append("Months=" + "=".join(str(item).lower() for item in months))

        if days is not None:
            scheduling.append("SpecificDays=true")
            # Generate a list with all values set to False, then overwrite the specified days with True
            day_list = []
            for i in range(31):
                day_list.append(False)

            for day in days:
                day_list[day-1] = True

            scheduling.append("Days=" + "=".join(str(item).lower() for item in day_list))
        else:
            scheduling.append("SpecificDays=false")

        return scheduling


    @staticmethod
    def generate_continuous_scheduler(start_time: str,
                                      time_zone: str = "America/New_York",
                                      restart_daily: bool = False,
                                      restart_error_count: int = 0,
                                      restart_error_delay_minutes: int = 0) -> List[str]:
        """
        Generate a scheduler array for a persistent query that has continuous scheduling.

        :param start_time: the query's restart time in HH:MM:SS format
        :param time_zone: the time zone, such as 'America/New_York' or 'Europe/London'
        :param restart_daily: if True, then the query should be restarted every day at the defined start_time
        :param restart_error_count: the number of times a failing query will attempt to be restarted within its run window
        :param restart_error_delay_minutes: the delay in minutes between attempting to restart a failed query
        :return: a scheduling definition array that can be passed to the controller when adding or updating a persistent query
        """

        if not restart_daily and start_time is None:
            start_time = "00:00:00"

        if restart_daily and start_time is None:
            raise ValueError("start_time must be provided when restart_daily is True")

        scheduling = GenerateScheduling._create_standard_parameters("com.illumon.iris.controller.IrisQuerySchedulerContinuous",
            start_time, None, time_zone, None, False, False, 0, False,
            True, restart_error_count, restart_error_delay_minutes, "Yes")
        scheduling.append("DailyRestart=" + str(restart_daily).lower())

        return scheduling


    @staticmethod
    def generate_range_scheduler(start_time: str,
                                 stop_time: str,
                                 start_date: str,
                                 end_date: str,
                                 time_zone: str = "America/New_York",
                                 repeat_enabled: bool = False,
                                 repeat_interval: int = 0,
                                 skip_if_unsuccessful: bool = False,
                                 restart_error_count: int = 0,
                                 restart_error_delay_minutes: int = 0,
                                 restart_when_running: str = "Yes") -> List[str]:
        """
        Generate a scheduler array for a persistent query that has range scheduling.

        :param start_time: the query's starting time in HH:MM:SS format, or None if no start date/time will be used
        :param stop_time: the query's stop time in HH:MM:SS format, or None if no stop date/time will be used
        :param start_date: the query's start date in YYYY-MM-DD format, or None if no start date/time will be used
        :param end_date: the query's end date in YYYY-MM-DD format, or None if no stop date/time will be used
        :param time_zone: the time zone, such as 'America/New_York' or 'Europe/London'
        :param repeat_enabled: for batch-style queries, if True, the persistent query will be restarted at the repeat_interval interval
        :param repeat_interval: if repeat_enabled is True, the interval in minutes between restarts
        :param skip_if_unsuccessful: for repeating queries, skip if the previous run failed when this is True
        :param restart_error_count: the number of times a failing query will attempt to be restarted within its run window
        :param restart_error_delay_minutes: the delay in minutes between attempting to restart a failed query
        :param restart_when_running: if 'Yes' then the query will be restarted when running at its start time, if 'No' it will be left running
        :return: a scheduling definition array that can be passed to the controller when adding or updating a persistent query
        """

        if start_time is not None and start_date is None:
            raise ValueError("if start_time is defined then start_date must be defined")

        if stop_time is not None and end_date is None:
            raise ValueError("if stop_time is defined then end_date must be defined")

        scheduling = GenerateScheduling._create_standard_parameters("com.illumon.iris.controller.IrisQuerySchedulerRange",
            start_time, stop_time, time_zone, None, False, repeat_enabled, repeat_interval, False,
            not repeat_enabled, restart_error_count, restart_error_delay_minutes, restart_when_running)

        if start_time is not None:
            scheduling.append("StartDate=" + start_date)
            scheduling.append("UseStartDateTime=true")
        else:
            scheduling.append("UseStartDateTime=false")

        if stop_time is not None:
            scheduling.append("StopDate=" + end_date)
            scheduling.append("UseStopDateTime=true")
        else:
            scheduling.append("UseStopDateTime=false")

        return scheduling


    @staticmethod
    def generate_temporary_scheduler(queue_name: str = "DefaultTemporaryQueue",
                                     restart_error_count: int = 0,
                                     restart_error_delay_minutes: int = 0,
                                     auto_delete: bool = True,
                                     expiration_time_minutes: int = 1440,
                                     dependent_query_serial: int = None) -> List[str]:
        """
        Generate a scheduler array for a persistent query that has temporary scheduling.

        :param queue_name: the temporary queue name
        :param restart_error_count: the number of times a failing query will attempt to be restarted within its run window
        :param restart_error_delay_minutes: the delay in minutes between attempting to restart a failed query
        :param auto_delete: if True, automatically delete the PQ after it's completed
        :param expiration_time_minutes: the time in minutes after which the temporary query will automatically be deleted (defaults to 1 day)
        :param dependent_query_serial: a single optional query on which the temporary query is dependent
        :return: a scheduling definition array that can be passed to the controller when adding or updating a persistent query
        """

        scheduling = GenerateScheduling._create_standard_parameters("com.illumon.iris.controller.IrisQuerySchedulerTemporary",
            "00:00:00", None, "America/New_York", None, False, False, 0, False,
            True, restart_error_count, restart_error_delay_minutes, "Yes")
        scheduling.append("TemporaryQueueName=" + queue_name)
        scheduling.append("TemporaryAutoDelete=" + str(auto_delete).lower())
        scheduling.append("TemporaryExpirationTimeMillis=" + str(expiration_time_minutes * 60000))
        if dependent_query_serial is not None:
            scheduling.append("TemporaryDependentQuerySerial=" + str(dependent_query_serial))

        return scheduling


    @staticmethod
    def generate_dependent_scheduler(start_time: str,
                                     stop_time: str,
                                     time_zone: str = "America/New_York",
                                     dependent_query_serials: List[int] = None,
                                     run_on_failure: bool = False,
                                     restart_on_condition: bool = False,
                                     run_on_any_dependency_met: bool = False,
                                     use_minimum_start_time: bool = False,
                                     run_each_time: bool = False,
                                     deadline_start: str = "00:00:00",
                                     deadline_end: str = "23:59:59",
                                     overnight: bool = False,
                                     repeat_enabled: bool = False,
                                     repeat_interval: int = 0,
                                     skip_if_unsuccessful: bool = False,
                                     stop_time_disabled: bool = False,
                                     restart_error_count: int = 0,
                                     restart_error_delay_minutes: int = 0,
                                     restart_when_running: str = "Yes") -> List[str]:
        """
        Generate a scheduler array for a persistent query that has dependent scheduling.

        :param start_time: the query's starting time in HH:MM:SS format
        :param stop_time: the query's stop time in HH:MM:SS format
        :param time_zone: the time zone, such as 'America/New_York' or 'Europe/London'
        :param dependent_query_serials: a list of serials on which the query is dependent
        :param run_on_failure: if True, run when dependencies fail (rather than succeed)
        :param restart_on_condition: if True, restart when conditions are met even if the query is running at the time
        :param run_on_any_dependency_met: if True, run when any dependency is met, otherwise all dependencies must be met
        :param use_minimum_start_time: if True, delay start of query after dependencies are met until the specified start_time
        :param run_each_time: if True, run every time dependencies are met, otherwise run once each day
        :param deadline_start: the start of the time at which the query should respond to dependency changes, in HH:MM:SS format
        :param deadline_end: the end of the time at which the query should respond to dependency changes, in HH:MM:SS format
        :param overnight: if True, the query runs overnight
        :param repeat_enabled: for batch-style queries, if True, the persistent query will be restarted at the repeat_interval interval
        :param repeat_interval: if repeat_enabled is True, the interval in minutes between restarts
        :param skip_if_unsuccessful: for repeating queries, skip if the previous run failed when this is True
        :param stop_time_disabled: if True, the persistent query does not need a stop time (for example, non-repeating batch queries)
        :param restart_error_count: the number of times a failing query will attempt to be restarted within its run window
        :param restart_error_delay_minutes: the delay in minutes between attempting to restart a failed query
        :param restart_when_running: if 'Yes' then the query will be restarted when running at its start time, if 'No' it will be left running
        :return: a scheduling definition array that can be passed to the controller when adding or updating a persistent query
        """

        if dependent_query_serials is None:
            raise ValueError("'dependent_query_serials' must be provided")

        if len(dependent_query_serials) < 1:
            raise ValueError("'dependent_query_serials' must contain at least one element")

        scheduling = GenerateScheduling._create_standard_parameters("com.illumon.iris.controller.IrisQuerySchedulerDependent",
            start_time, stop_time, time_zone, None, overnight, repeat_enabled, repeat_interval, skip_if_unsuccessful,
            stop_time_disabled, restart_error_count, restart_error_delay_minutes, restart_when_running)
        scheduling.append("DependentQuerySerial=" + ";".join(str(item) for item in dependent_query_serials))
        scheduling.append("RunOnFailure=" + str(run_on_failure).lower())
        scheduling.append("RestartOnCondition=" + str(restart_on_condition).lower())
        scheduling.append("RunOnAny=" + str(run_on_any_dependency_met).lower())
        scheduling.append("UseMinStartTime=" + str(use_minimum_start_time).lower())
        scheduling.append("RunEachTime=" + str(run_each_time).lower())
        scheduling.append("DeadlineStart=" + deadline_start)
        scheduling.append("DeadlineEnd=" + deadline_end)

        return scheduling


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deephaven_enterprise/proto/common.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'deephaven_enterprise/proto/common.proto\x12\x19io.deephaven.proto.common\"\x1c\n\x0b\x41uthRequest\x12\r\n\x05token\x18\x01 \x01(\x0c\"$\n\tAuthReply\x12\x17\n\x0fisAuthenticated\x18\x01 \x01(\x08\"\x0e\n\x0c\x41\x62outRequest\"<\n\nAboutReply\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\"<\n\x0bPingRequest\x12\x1f\n\x17sender_send_time_millis\x18\x01 \x01(\x10\x12\x0c\n\x04\x66rom\x18\x02 \x01(\t\"B\n\x0cPingResponse\x12$\n\x1creceiver_receive_time_millis\x18\x01 \x01(\x10\x12\x0c\n\x04\x66rom\x18\x02 \x01(\t\"X\n\x17\x45xceptionDetailsMessage\x12\x14\n\x0c\x65rrorMessage\x18\x01 \x01(\t\x12\x12\n\nstackTrace\x18\x02 \x01(\t\x12\x13\n\x0bshortCauses\x18\x03 \x01(\t*3\n\x10NamespaceSetEnum\x12\t\n\x05UNSET\x10\x00\x12\n\n\x06SYSTEM\x10\x01\x12\x08\n\x04USER\x10\x02\x42\x04H\x01P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deephaven_enterprise.proto.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\001P\001'
  _globals['_NAMESPACESETENUM']._serialized_start=436
  _globals['_NAMESPACESETENUM']._serialized_end=487
  _globals['_AUTHREQUEST']._serialized_start=70
  _globals['_AUTHREQUEST']._serialized_end=98
  _globals['_AUTHREPLY']._serialized_start=100
  _globals['_AUTHREPLY']._serialized_end=136
  _globals['_ABOUTREQUEST']._serialized_start=138
  _globals['_ABOUTREQUEST']._serialized_end=152
  _globals['_ABOUTREPLY']._serialized_start=154
  _globals['_ABOUTREPLY']._serialized_end=214
  _globals['_PINGREQUEST']._serialized_start=216
  _globals['_PINGREQUEST']._serialized_end=276
  _globals['_PINGRESPONSE']._serialized_start=278
  _globals['_PINGRESPONSE']._serialized_end=344
  _globals['_EXCEPTIONDETAILSMESSAGE']._serialized_start=346
  _globals['_EXCEPTIONDETAILSMESSAGE']._serialized_end=434
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deephaven_enterprise/proto/acl.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from deephaven_enterprise.proto import auth_pb2 as deephaven__enterprise_dot_proto_dot_auth__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$deephaven_enterprise/proto/acl.proto\x12\x1dio.deephaven.proto.controller\x1a%deephaven_enterprise/proto/auth.proto\"V\n\x10UnaryUserRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x32\n\x04user\x18\x02 \x01(\x0b\x32$.io.deephaven.proto.auth.UserContext\"2\n\x11UnaryGroupRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\r\n\x05group\x18\x02 \x01(\t\"!\n\x0f\x42ooleanResponse\x12\x0e\n\x06isTrue\x18\x01 \x01(\x08\"8\n\x0fUserOrGroupList\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x15\n\rusersOrGroups\x18\x02 \x03(\t\"$\n\x12GetAllUsersRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\"%\n\x13GetAllGroupsRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x42\x04H\x01P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deephaven_enterprise.proto.acl_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\001P\001'
  _globals['_UNARYUSERREQUEST']._serialized_start=110
  _globals['_UNARYUSERREQUEST']._serialized_end=196
  _globals['_UNARYGROUPREQUEST']._serialized_start=198
  _globals['_UNARYGROUPREQUEST']._serialized_end=248
  _globals['_BOOLEANRESPONSE']._serialized_start=250
  _globals['_BOOLEANRESPONSE']._serialized_end=283
  _globals['_USERORGROUPLIST']._serialized_start=285
  _globals['_USERORGROUPLIST']._serialized_end=341
  _globals['_GETALLUSERSREQUEST']._serialized_start=343
  _globals['_GETALLUSERSREQUEST']._serialized_end=379
  _globals['_GETALLGROUPSREQUEST']._serialized_start=381
  _globals['_GETALLGROUPSREQUEST']._serialized_end=418
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deephaven_enterprise/proto/table_definition.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1deephaven_enterprise/proto/table_definition.proto\x12\x19io.deephaven.proto.common\"\x9c\x03\n\x17\x43olumnDefinitionMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61taType\x18\x02 \x01(\t\x12\x1a\n\rcomponentType\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x42\n\ncolumnType\x18\x04 \x01(\x0e\x32).io.deephaven.proto.common.ColumnTypeEnumH\x01\x88\x01\x01\x12\x17\n\x0fisVarSizeString\x18\x05 \x01(\x08\x12\x42\n\x08\x65ncoding\x18\x06 \x01(\x0e\x32+.io.deephaven.proto.common.EncodingTypeEnumH\x02\x88\x01\x01\x12\x12\n\x05\x63odec\x18\x07 \x01(\tH\x03\x88\x01\x01\x12\x16\n\tcodecArgs\x18\x08 \x01(\tH\x04\x88\x01\x01\x12\x1d\n\x10objectWidthBytes\x18\t \x01(\x05H\x05\x88\x01\x01\x42\x10\n\x0e_componentTypeB\r\n\x0b_columnTypeB\x0b\n\t_encodingB\x08\n\x06_codecB\x0c\n\n_codecArgsB\x13\n\x11_objectWidthBytes\"\xff\x01\n\x16TableDefinitionMessage\x12\x16\n\tnamespace\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x16\n\ttableName\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x43\n\x07\x63olumns\x18\x03 \x03(\x0b\x32\x32.io.deephaven.proto.common.ColumnDefinitionMessage\x12\x44\n\x0bstorageType\x18\x04 \x01(\x0e\x32*.io.deephaven.proto.common.StorageTypeEnumH\x02\x88\x01\x01\x42\x0c\n\n_namespaceB\x0c\n\n_tableNameB\x0e\n\x0c_storageType*\x83\x01\n\x0fStorageTypeEnum\x12\x12\n\x0eST_UNSPECIFIED\x10\x00\x12\x10\n\x0cST_IN_MEMORY\x10\x01\x12!\n\x1dST_NESTED_PARTITIONED_ON_DISK\x10\x02\x12\x16\n\x12ST_SPLAYED_ON_DISK\x10\x04\x12\x0f\n\x0bST_EXTENDED\x10\x08*i\n\x0e\x43olumnTypeEnum\x12\x12\n\x0e\x43T_UNSPECIFIED\x10\x00\x12\r\n\tCT_NORMAL\x10\x01\x12\x0f\n\x0b\x43T_GROUPING\x10\x02\x12\x13\n\x0f\x43T_PARTITIONING\x10\x04\x12\x0e\n\nCT_VIRTUAL\x10\x08*\x89\x01\n\x10\x45ncodingTypeEnum\x12\x12\n\x0e\x45T_UNSPECIFIED\x10\x00\x12\x11\n\rET_ISO_8859_1\x10\x01\x12\x0c\n\x08\x45T_UTF_8\x10\x02\x12\x0f\n\x0b\x45T_US_ASCII\x10\x04\x12\r\n\tET_UTF_16\x10\x08\x12\x0f\n\x0b\x45T_UTF_16BE\x10\x10\x12\x0f\n\x0b\x45T_UTF_16LE\x10 B\x04H\x01P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deephaven_enterprise.proto.table_definition_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\001P\001'
  _globals['_STORAGETYPEENUM']._serialized_start=754
  _globals['_STORAGETYPEENUM']._serialized_end=885
  _globals['_COLUMNTYPEENUM']._serialized_start=887
  _globals['_COLUMNTYPEENUM']._serialized_end=992
  _globals['_ENCODINGTYPEENUM']._serialized_start=995
  _globals['_ENCODINGTYPEENUM']._serialized_end=1132
  _globals['_COLUMNDEFINITIONMESSAGE']._serialized_start=81
  _globals['_COLUMNDEFINITIONMESSAGE']._serialized_end=493
  _globals['_TABLEDEFINITIONMESSAGE']._serialized_start=496
  _globals['_TABLEDEFINITIONMESSAGE']._serialized_end=751
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deephaven_enterprise/proto/controller_service.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from deephaven_enterprise.proto import controller_common_pb2 as deephaven__enterprise_dot_proto_dot_controller__common__pb2
from deephaven_enterprise.proto import controller_pb2 as deephaven__enterprise_dot_proto_dot_controller__pb2
from deephaven_enterprise.proto import acl_pb2 as deephaven__enterprise_dot_proto_dot_acl__pb2
from deephaven_enterprise.proto import auth_pb2 as deephaven__enterprise_dot_proto_dot_auth__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3deephaven_enterprise/proto/controller_service.proto\x12\"io.deephaven.proto.controller.grpc\x1a\x32\x64\x65\x65phaven_enterprise/proto/controller_common.proto\x1a+deephaven_enterprise/proto/controller.proto\x1a$deephaven_enterprise/proto/acl.proto\x1a%deephaven_enterprise/proto/auth.proto2\xd6\x12\n\rControllerApi\x12}\n\x0c\x61uthenticate\x12\x34.io.deephaven.proto.controller.AuthenticationRequest\x1a\x35.io.deephaven.proto.controller.AuthenticationResponse\"\x00\x12m\n\x08\x61\x64\x64Query\x12..io.deephaven.proto.controller.AddQueryRequest\x1a/.io.deephaven.proto.controller.AddQueryResponse\"\x00\x12v\n\x0bremoveQuery\x12\x31.io.deephaven.proto.controller.RemoveQueryRequest\x1a\x32.io.deephaven.proto.controller.RemoveQueryResponse\"\x00\x12v\n\x0bmodifyQuery\x12\x31.io.deephaven.proto.controller.ModifyQueryRequest\x1a\x32.io.deephaven.proto.controller.ModifyQueryResponse\"\x00\x12y\n\x0crestartQuery\x12\x32.io.deephaven.proto.controller.RestartQueryRequest\x1a\x33.io.deephaven.proto.controller.RestartQueryResponse\"\x00\x12p\n\tstopQuery\x12/.io.deephaven.proto.controller.StopQueryRequest\x1a\x30.io.deephaven.proto.controller.StopQueryResponse\"\x00\x12\x7f\n\x0erestartReplica\x12\x34.io.deephaven.proto.controller.RestartReplicaRequest\x1a\x35.io.deephaven.proto.controller.RestartReplicaResponse\"\x00\x12v\n\x0bstopReplica\x12\x31.io.deephaven.proto.controller.StopReplicaRequest\x1a\x32.io.deephaven.proto.controller.StopReplicaResponse\"\x00\x12\x8e\x01\n\x13\x64\x65termineDispatcher\x12\x39.io.deephaven.proto.controller.DetermineDispatcherRequest\x1a:.io.deephaven.proto.controller.DetermineDispatcherResponse\"\x00\x12\x96\x01\n\x17selectionProviderStatus\x12;.io.deephaven.proto.controller.ServerSelectionStatusRequest\x1a<.io.deephaven.proto.controller.ServerSelectionStatusResponse\"\x00\x12\x93\x01\n\x16selectionProviderAdmin\x12:.io.deephaven.proto.controller.ServerSelectionAdminRequest\x1a;.io.deephaven.proto.controller.ServerSelectionAdminResponse\"\x00\x12\x85\x01\n\x10getConfiguration\x12\x36.io.deephaven.proto.controller.GetConfigurationRequest\x1a\x37.io.deephaven.proto.controller.GetConfigurationResponse\"\x00\x12\x7f\n\x0egetScriptPaths\x12\x34.io.deephaven.proto.controller.GetScriptPathsRequest\x1a\x35.io.deephaven.proto.controller.GetScriptPathsResponse\"\x00\x12p\n\tgetScript\x12/.io.deephaven.proto.controller.GetScriptRequest\x1a\x30.io.deephaven.proto.controller.GetScriptResponse\"\x00\x12y\n\x0cstatusUpdate\x12\x32.io.deephaven.proto.controller.StatusUpdateRequest\x1a\x33.io.deephaven.proto.controller.StatusUpdateResponse\"\x00\x12y\n\x0creloadConfig\x12\x32.io.deephaven.proto.controller.ReloadConfigRequest\x1a\x33.io.deephaven.proto.controller.ReloadConfigResponse\"\x00\x12\x61\n\x04ping\x12*.io.deephaven.proto.controller.PingRequest\x1a+.io.deephaven.proto.controller.PingResponse\"\x00\x12r\n\tsubscribe\x12/.io.deephaven.proto.controller.SubscribeRequest\x1a\x30.io.deephaven.proto.controller.SubscribeResponse\"\x00\x30\x01\x12y\n\x10invalidateCookie\x12\x30.io.deephaven.proto.auth.InvalidateCookieRequest\x1a\x31.io.deephaven.proto.auth.InvalidateCookieResponse\"\x00\x32\xb0\x07\n\x06\x41\x63lApi\x12p\n\x0bisSuperUser\x12/.io.deephaven.proto.controller.UnaryUserRequest\x1a..io.deephaven.proto.controller.BooleanResponse\"\x00\x12p\n\x0bisAclEditor\x12/.io.deephaven.proto.controller.UnaryUserRequest\x1a..io.deephaven.proto.controller.BooleanResponse\"\x00\x12s\n\x0eisQueryManager\x12/.io.deephaven.proto.controller.UnaryUserRequest\x1a..io.deephaven.proto.controller.BooleanResponse\"\x00\x12t\n\x0fisSchemaManager\x12/.io.deephaven.proto.controller.UnaryUserRequest\x1a..io.deephaven.proto.controller.BooleanResponse\"\x00\x12v\n\x10getUsersForGroup\x12\x30.io.deephaven.proto.controller.UnaryGroupRequest\x1a..io.deephaven.proto.controller.UserOrGroupList\"\x00\x12u\n\x10getGroupsForUser\x12/.io.deephaven.proto.controller.UnaryUserRequest\x1a..io.deephaven.proto.controller.UserOrGroupList\"\x00\x12r\n\x0bgetAllUsers\x12\x31.io.deephaven.proto.controller.GetAllUsersRequest\x1a..io.deephaven.proto.controller.UserOrGroupList\"\x00\x12t\n\x0cgetAllGroups\x12\x32.io.deephaven.proto.controller.GetAllGroupsRequest\x1a..io.deephaven.proto.controller.UserOrGroupList\"\x00\x42\x04H\x01P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deephaven_enterprise.proto.controller_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\001P\001'
  _globals['_CONTROLLERAPI']._serialized_start=266
  _globals['_CONTROLLERAPI']._serialized_end=2656
  _globals['_ACLAPI']._serialized_start=2659
  _globals['_ACLAPI']._serialized_end=3603
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import binascii
import grpc
import google.protobuf
import socket
import time
import uuid

import deephaven_enterprise.proto.auth_pb2 as auth_pb2


"""
Utility functions for the Deephaven Enterprise client, these are not part of the stable API or intended for external use. 
"""
def _proto_to_key(proto: google.protobuf.message.Message) -> str:
    return proto.DESCRIPTOR.full_name.lower()+"-bin"


_auth_error_metadata_key: str = _proto_to_key(auth_pb2.AuthError())


def get_grpc_channel(channel: grpc.Channel, host: str, port: int, channel_options=None) -> (grpc.Channel, str):
    """
    Given a channel or host and port, produce the grpc channel for use with the auth client or controller client and a
    "me" string that is used for logging on the server.

    :param channel: an already configured grpc channel (mutually exclusive with host and port)
    :param host: the host name to connect to (requires port)
    :param port: the port to connect to (requires host)
    :param channel_options: a list of options for grpc channel creation (not used when channel is set)
    :return:
    """
    if channel is not None:
        return (channel, "Python:" + get_ip("127.0.0.1", 80))
    elif host is not None:
        if port is None:
            raise Exception("port is required when host is specified")

        me = "Python:" + get_ip(host, port)
        target = host + ":" + str(port)
        channel = grpc.secure_channel(
            target, grpc.ssl_channel_credentials(), options=channel_options)
        return (channel, me)
    else:
        raise Exception("channel or host is required")


def make_client_id(me: str) -> auth_pb2.ClientId:
    """
    Generate a Deephaven client id using the "me" description and a random UUID.
    :param me: client identifier
    :return: a ClientID for use in RPCs
    """
    clientID: auth_pb2.ClientId = auth_pb2.ClientId()
    clientID.uuid = uuid.uuid4().bytes
    clientID.name = me
    return clientID


def _uuid_str(bs: bytes) -> str:
    return "UUID=" + binascii.hexlify(bs).decode('utf-8')


def get_ip(host: str, port: int) -> str:
    """
    Gets the local address by creating a datagram socket to the specified host and port and requesting the local sockname.
    :param host:remote host
    :param port:remote port
    :return: the local ip address (or "<unknown>")
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(0)
    # noinspection PyBroadException
    try:
        # doesn't even have to be reachable
        s.connect((host, port))
        ip = s.getsockname()[0]
    except Exception:
        ip = '<unknown>'
    finally:
        s.close()
    return ip


def _maybe_reinterpret_and_raise(error: grpc.RpcError):
    auth_error = auth_pb2.AuthError()
    if len(error.trailing_metadata()) > 0:
        for md in error.trailing_metadata():
            if md.key == _auth_error_metadata_key:
                auth_error.ParseFromString(md.value)
                raise Exception(auth_error.message) from None
    raise error



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deephaven_enterprise/proto/controller.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from deephaven_enterprise.proto import auth_pb2 as deephaven__enterprise_dot_proto_dot_auth__pb2
from deephaven_enterprise.proto import controller_common_pb2 as deephaven__enterprise_dot_proto_dot_controller__common__pb2
from deephaven_enterprise.proto import persistent_query_pb2 as deephaven__enterprise_dot_proto_dot_persistent__query__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+deephaven_enterprise/proto/controller.proto\x12\x1dio.deephaven.proto.controller\x1a%deephaven_enterprise/proto/auth.proto\x1a\x32\x64\x65\x65phaven_enterprise/proto/controller_common.proto\x1a\x31\x64\x65\x65phaven_enterprise/proto/persistent_query.proto\"\x95\x01\n\x15\x41uthenticationRequest\x12-\n\x05token\x18\x01 \x01(\x0b\x32\x1e.io.deephaven.proto.auth.Token\x12\x33\n\x08\x63lientId\x18\x02 \x01(\x0b\x32!.io.deephaven.proto.auth.ClientId\x12\x18\n\x10getConfiguration\x18\x03 \x01(\x08\"\x9e\x01\n\x16\x41uthenticationResponse\x12\x15\n\rauthenticated\x18\x01 \x01(\x08\x12\x0e\n\x06\x63ookie\x18\x02 \x01(\x0c\x12R\n\x06\x63onfig\x18\x03 \x01(\x0b\x32=.io.deephaven.proto.controller.ControllerConfigurationMessageH\x00\x88\x01\x01\x42\t\n\x07_config\"n\n\x0f\x41\x64\x64QueryRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12K\n\x06\x63onfig\x18\x02 \x01(\x0b\x32;.io.deephaven.proto.controller.PersistentQueryConfigMessage\"\'\n\x10\x41\x64\x64QueryResponse\x12\x13\n\x0bquerySerial\x18\x01 \x01(\x03\"4\n\x12RemoveQueryRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x0e\n\x06serial\x18\x02 \x01(\x03\"\x15\n\x13RemoveQueryResponse\"\x84\x01\n\x12ModifyQueryRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12K\n\x06\x63onfig\x18\x02 \x01(\x0b\x32;.io.deephaven.proto.controller.PersistentQueryConfigMessage\x12\x11\n\tdoRestart\x18\x03 \x01(\x08\"\x15\n\x13ModifyQueryResponse\"z\n\x10ReplicaSpecifier\x12\x0e\n\x06serial\x18\x01 \x01(\x03\x12\x11\n\x07replica\x18\x02 \x01(\x05H\x00\x12\x0f\n\x05spare\x18\x03 \x01(\x05H\x00\x12\x1a\n\rprocessInfoId\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x04\n\x02idB\x10\n\x0e_processInfoId\"\x80\x01\n\rReplicaStatus\x12@\n\x07replica\x18\x01 \x01(\x0b\x32/.io.deephaven.proto.controller.ReplicaSpecifier\x12\x0f\n\x07success\x18\x02 \x01(\x08\x12\x12\n\x05\x65rror\x18\x03 \x01(\tH\x00\x88\x01\x01\x42\x08\n\x06_error\"j\n\x15RestartReplicaRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x41\n\x08replicas\x18\x02 \x03(\x0b\x32/.io.deephaven.proto.controller.ReplicaSpecifier\"V\n\x16RestartReplicaResponse\x12<\n\x06status\x18\x01 \x03(\x0b\x32,.io.deephaven.proto.controller.ReplicaStatus\"g\n\x12StopReplicaRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x41\n\x08replicas\x18\x02 \x03(\x0b\x32/.io.deephaven.proto.controller.ReplicaSpecifier\"S\n\x13StopReplicaResponse\x12<\n\x06status\x18\x01 \x03(\x0b\x32,.io.deephaven.proto.controller.ReplicaStatus\"6\n\x13RestartQueryRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x0f\n\x07serials\x18\x02 \x03(\x03\"\xc0\x01\n\x14RestartQueryResponse\x12\x16\n\x0esuccessSerials\x18\x01 \x03(\x03\x12[\n\x0c\x65rrorSerials\x18\x02 \x03(\x0b\x32\x45.io.deephaven.proto.controller.RestartQueryResponse.ErrorSerialsEntry\x1a\x33\n\x11\x45rrorSerialsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"3\n\x10StopQueryRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x0f\n\x07serials\x18\x02 \x03(\x03\"\xba\x01\n\x11StopQueryResponse\x12\x16\n\x0esuccessSerials\x18\x01 \x03(\x03\x12X\n\x0c\x65rrorSerials\x18\x02 \x03(\x0b\x32\x42.io.deephaven.proto.controller.StopQueryResponse.ErrorSerialsEntry\x1a\x33\n\x11\x45rrorSerialsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x03\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"m\n\x1a\x44\x65termineDispatcherRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x17\n\x0fserverGroupName\x18\x02 \x01(\t\x12\x12\n\nheapSizeMb\x18\x03 \x01(\x05\x12\x12\n\nworkerKind\x18\x04 \x01(\t\"k\n\x1b\x44\x65termineDispatcherResponse\x12L\n\x0cserverConfig\x18\x01 \x01(\x0b\x32\x36.io.deephaven.proto.controller.PQDBServerConfigMessage\"\xb7\x02\n\x17PQDBServerConfigMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04host\x18\x02 \x01(\t\x12\x0c\n\x04port\x18\x03 \x01(\x05\x12&\n\x19\x63lassPushListResourceName\x18\x04 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bserverClass\x18\x05 \x01(\t\x12\x15\n\rmaxHeapSizeGb\x18\x06 \x01(\x01\x12\x41\n\nserverType\x18\x07 \x01(\x0e\x32-.io.deephaven.proto.controller.ServerTypeEnum\x12\x15\n\rconsoleGroups\x18\x08 \x03(\t\x12\x17\n\nserverInfo\x18\t \x01(\tH\x01\x88\x01\x01\x42\x1c\n\x1a_classPushListResourceNameB\r\n\x0b_serverInfo\"\x87\x01\n\x1d\x41ssignmentPolicyConfigMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64isplayName\x18\x02 \x01(\t\x12\x18\n\x0b\x64\x65scription\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x19\n\x11\x61\x63\x63\x65ptsParameters\x18\x04 \x01(\x08\x42\x0e\n\x0c_description\")\n\x17GetConfigurationRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\"i\n\x18GetConfigurationResponse\x12M\n\x06\x63onfig\x18\x01 \x01(\x0b\x32=.io.deephaven.proto.controller.ControllerConfigurationMessage\"\xc7\x08\n\x1e\x43ontrollerConfigurationMessage\x12\x15\n\rmaxHeapSizeGb\x18\x01 \x01(\x05\x12\x1e\n\x16minDataBufferPoolRatio\x18\x02 \x01(\x01\x12\x1e\n\x16maxDataBufferPoolRatio\x18\x03 \x01(\x01\x12Z\n\x12\x63onfigurationTypes\x18\x04 \x03(\x0b\x32>.io.deephaven.proto.controller.ConsoleConfigurationTypeMessage\x12_\n\x15temporaryQueueDetails\x18\x05 \x03(\x0b\x32@.io.deephaven.proto.controller.ControllerTempQueueDetailsMessage\x12\x1c\n\x14\x64\x65\x66\x61ultTempQueueName\x18\x06 \x01(\t\x12I\n\tdbServers\x18\x07 \x03(\x0b\x32\x36.io.deephaven.proto.controller.PQDBServerConfigMessage\x12 \n\x18\x64\x65\x66\x61ultSchedulingDetails\x18\x08 \x03(\t\x12\x1f\n\x17scheduleLookForwardDays\x18\t \x01(\x05\x12\x11\n\tpublicKey\x18\n \x01(\x0c\x12\x11\n\talgorithm\x18\x0b \x01(\t\x12K\n\x13\x63onsoleServerGroups\x18\x0c \x03(\x0b\x32..io.deephaven.proto.controller.NamedStringList\x12\x17\n\x0fjvmProfileNames\x18\r \x03(\t\x12\x1d\n\x15\x64\x65\x66\x61ultJvmProfileName\x18\x0e \x01(\t\x12\x1b\n\x13\x64\x61taRoutingImplName\x18\x0f \x01(\t\x12G\n\ntdsConfigs\x18\x10 \x03(\x0b\x32\x33.io.deephaven.proto.controller.RoutingConfigMessage\x12G\n\ndisConfigs\x18\x11 \x03(\x0b\x32\x33.io.deephaven.proto.controller.RoutingConfigMessage\x12G\n\nlasConfigs\x18\x12 \x03(\x0b\x32\x33.io.deephaven.proto.controller.RoutingConfigMessage\x12X\n\x12\x61ssignmentPolicies\x18\x14 \x03(\x0b\x32<.io.deephaven.proto.controller.AssignmentPolicyConfigMessage\x12#\n\x1b\x64\x65\x66\x61ultAssignmentPolicyName\x18\x15 \x01(\t\x12%\n\x1d\x64\x65\x66\x61ultAssignmentPolicyParams\x18\x16 \x01(\tJ\x04\x08\x13\x10\x14R\x16inWorkerServiceDetails\"G\n\x14RoutingConfigMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04tags\x18\x02 \x03(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\"b\n!ControllerTempQueueDetailsMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x1c\n\x14maxConcurrentQueries\x18\x02 \x01(\x05\x12\x11\n\tmaxHeapMb\x18\x03 \x01(\x05\"\x85\x07\n\x1f\x43onsoleConfigurationTypeMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64isplayName\x18\x02 \x01(\t\x12\x13\n\x0b\x64isplayable\x18\x03 \x01(\x08\x12\x62\n\nproperties\x18\x04 \x03(\x0b\x32N.io.deephaven.proto.controller.ConsoleConfigurationTypeMessage.PropertiesEntry\x12\x11\n\thasScript\x18\x05 \x01(\x08\x12\x15\n\rserverClasses\x18\x06 \x03(\t\x12\x15\n\rallowedGroups\x18\x07 \x03(\t\x12G\n\x0c\x65xtraColumns\x18\x08 \x03(\x0b\x32\x31.io.deephaven.proto.controller.ExtraColumnMessage\x12\'\n\x1atypeSpecificWireConfigJSON\x18\t \x01(\tH\x00\x88\x01\x01\x12\x1a\n\x12\x63onfigCheckerClass\x18\n \x01(\t\x12\x1f\n\x17\x63onfigPanelFactoryClass\x18\x0b \x01(\t\x12!\n\x19\x63ustomActionProviderClass\x18\x0c \x01(\t\x12\x17\n\x0fsetupQueryClass\x18\r \x01(\t\x12\x13\n\x0b\x63onfigClass\x18\x0e \x01(\t\x12\x1e\n\x16shutdownProcedureClass\x18\x0f \x01(\t\x12\x19\n\x11\x65xtraJvmArgsClass\x18\x10 \x01(\t\x12\x1c\n\x14\x63onfigProcessorClass\x18\x11 \x01(\t\x12\x1e\n\x16\x65xtraColumnGetterClass\x18\x12 \x01(\t\x12\x1a\n\x12popupProviderClass\x18\x13 \x03(\t\x12\x1e\n\x16\x63onfigTypeDetailsClass\x18\x14 \x01(\t\x12!\n\x19\x63ommunityInitializerClass\x18\x15 \x01(\t\x12\x19\n\x11hasScriptLanguage\x18\x16 \x01(\x08\x12&\n\x1e\x63ommunityRequestProcessorClass\x18\x17 \x01(\t\x12\x18\n\x10supportsReplicas\x18\x18 \x01(\x08\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\x1d\n\x1b_typeSpecificWireConfigJSON\"8\n\x12\x45xtraColumnMessage\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0crendererName\x18\x02 \x01(\t\".\n\x1cServerSelectionStatusRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\"/\n\x1dServerSelectionStatusResponse\x12\x0e\n\x06status\x18\x01 \x01(\t\">\n\x1bServerSelectionAdminRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x0f\n\x07\x63ommand\x18\x02 \x01(\t\".\n\x1cServerSelectionAdminResponse\x12\x0e\n\x06result\x18\x01 \x01(\t\"\x99\x02\n\x15GetScriptPathsRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x41\n\x07\x63ommand\x18\x02 \x01(\x0e\x32\x30.io.deephaven.proto.controller.ScriptCommandEnum\x12\x18\n\x0bquerySerial\x18\x03 \x01(\x03H\x00\x88\x01\x01\x12\x1a\n\reffectiveUser\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\"\n\x15scriptLoaderStateJSON\x18\x05 \x01(\tH\x02\x88\x01\x01\x12\x17\n\x0fuseRelativePath\x18\x06 \x01(\x08\x42\x0e\n\x0c_querySerialB\x10\n\x0e_effectiveUserB\x18\n\x16_scriptLoaderStateJSON\")\n\x16GetScriptPathsResponse\x12\x0f\n\x07results\x18\x01 \x03(\t\"\xa8\x02\n\x10GetScriptRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x41\n\x07\x63ommand\x18\x02 \x01(\x0e\x32\x30.io.deephaven.proto.controller.ScriptCommandEnum\x12\x18\n\x0bquerySerial\x18\x03 \x01(\x03H\x00\x88\x01\x01\x12\x1a\n\reffectiveUser\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\"\n\x15scriptLoaderStateJSON\x18\x05 \x01(\tH\x02\x88\x01\x01\x12\x12\n\nscriptPath\x18\x06 \x01(\t\x12\x17\n\x0fuseRelativePath\x18\x07 \x01(\x08\x42\x0e\n\x0c_querySerialB\x10\n\x0e_effectiveUserB\x18\n\x16_scriptLoaderStateJSON\";\n\x11GetScriptResponse\x12\x17\n\nscriptBody\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_scriptBody\"\xf7\x02\n\x13StatusUpdateRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\x12\x13\n\x0bquerySerial\x18\x02 \x01(\x03\x12M\n\x06status\x18\x03 \x01(\x0e\x32\x38.io.deephaven.proto.controller.PersistentQueryStatusEnumH\x00\x88\x01\x01\x12\x1a\n\x12hasExportedObjects\x18\x04 \x01(\x08\x12Q\n\x0f\x65xportedObjects\x18\x05 \x03(\x0b\x32\x38.io.deephaven.proto.controller.ExportedObjectInfoMessage\x12\x16\n\x0ehasTableGroups\x18\x06 \x01(\x08\x12\x43\n\x0btableGroups\x18\x07 \x03(\x0b\x32..io.deephaven.proto.controller.NamedStringList\x12\x15\n\rprocessInfoId\x18\x08 \x01(\tB\t\n\x07_status\"\x16\n\x14StatusUpdateResponse\"%\n\x13ReloadConfigRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\"\x16\n\x14ReloadConfigResponse\"\"\n\x10SubscribeRequest\x12\x0e\n\x06\x63ookie\x18\x01 \x01(\x0c\"\x94\x02\n\x11SubscribeResponse\x12?\n\x05\x65vent\x18\x01 \x01(\x0e\x32\x30.io.deephaven.proto.controller.SubscriptionEvent\x12\x15\n\x0bquerySerial\x18\x03 \x01(\x03H\x00\x12N\n\tqueryInfo\x18\x04 \x01(\x0b\x32\x39.io.deephaven.proto.controller.PersistentQueryInfoMessageH\x00\x12O\n\x06\x63onfig\x18\x05 \x01(\x0b\x32=.io.deephaven.proto.controller.ControllerConfigurationMessageH\x00\x42\x06\n\x04\x64\x61ta*H\n\x0eServerTypeEnum\x12\x13\n\x0fSTE_UNSPECIFIED\x10\x00\x12\x12\n\x0eSTE_DISPATCHER\x10\x01\x12\r\n\tSTE_GROUP\x10\x02*j\n\x11ScriptCommandEnum\x12\x13\n\x0fSCE_UNSPECIFIED\x10\x00\x12\x13\n\x0fSCE_GET_AS_SELF\x10\x01\x12\x14\n\x10SCE_GET_AS_OTHER\x10\x02\x12\x15\n\x11SCE_GET_FOR_QUERY\x10\x03*j\n\x11SubscriptionEvent\x12\x12\n\x0eSE_UNSPECIFIED\x10\x00\x12\n\n\x06SE_PUT\x10\x01\x12\r\n\tSE_REMOVE\x10\x02\x12\x14\n\x10SE_CONFIG_UPDATE\x10\x03\x12\x10\n\x0cSE_BATCH_END\x10\x04\x42\x04H\x01P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deephaven_enterprise.proto.controller_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\001P\001'
  _globals['_RESTARTQUERYRESPONSE_ERRORSERIALSENTRY']._loaded_options = None
  _globals['_RESTARTQUERYRESPONSE_ERRORSERIALSENTRY']._serialized_options = b'8\001'
  _globals['_STOPQUERYRESPONSE_ERRORSERIALSENTRY']._loaded_options = None
  _globals['_STOPQUERYRESPONSE_ERRORSERIALSENTRY']._serialized_options = b'8\001'
  _globals['_CONSOLECONFIGURATIONTYPEMESSAGE_PROPERTIESENTRY']._loaded_options = None
  _globals['_CONSOLECONFIGURATIONTYPEMESSAGE_PROPERTIESENTRY']._serialized_options = b'8\001'
  _globals['_SERVERTYPEENUM']._serialized_start=6786
  _globals['_SERVERTYPEENUM']._serialized_end=6858
  _globals['_SCRIPTCOMMANDENUM']._serialized_start=6860
  _globals['_SCRIPTCOMMANDENUM']._serialized_end=6966
  _globals['_SUBSCRIPTIONEVENT']._serialized_start=6968
  _globals['_SUBSCRIPTIONEVENT']._serialized_end=7074
  _globals['_AUTHENTICATIONREQUEST']._serialized_start=221
  _globals['_AUTHENTICATIONREQUEST']._serialized_end=370
  _globals['_AUTHENTICATIONRESPONSE']._serialized_start=373
  _globals['_AUTHENTICATIONRESPONSE']._serialized_end=531
  _globals['_ADDQUERYREQUEST']._serialized_start=533
  _globals['_ADDQUERYREQUEST']._serialized_end=643
  _globals['_ADDQUERYRESPONSE']._serialized_start=645
  _globals['_ADDQUERYRESPONSE']._serialized_end=684
  _globals['_REMOVEQUERYREQUEST']._serialized_start=686
  _globals['_REMOVEQUERYREQUEST']._serialized_end=738
  _globals['_REMOVEQUERYRESPONSE']._serialized_start=740
  _globals['_REMOVEQUERYRESPONSE']._serialized_end=761
  _globals['_MODIFYQUERYREQUEST']._serialized_start=764
  _globals['_MODIFYQUERYREQUEST']._serialized_end=896
  _globals['_MODIFYQUERYRESPONSE']._serialized_start=898
  _globals['_MODIFYQUERYRESPONSE']._serialized_end=919
  _globals['_REPLICASPECIFIER']._serialized_start=921
  _globals['_REPLICASPECIFIER']._serialized_end=1043
  _globals['_REPLICASTATUS']._serialized_start=1046
  _globals['_REPLICASTATUS']._serialized_end=1174
  _globals['_RESTARTREPLICAREQUEST']._serialized_start=1176
  _globals['_RESTARTREPLICAREQUEST']._serialized_end=1282
  _globals['_RESTARTREPLICARESPONSE']._serialized_start=1284
  _globals['_RESTARTREPLICARESPONSE']._serialized_end=1370
  _globals['_STOPREPLICAREQUEST']._serialized_start=1372
  _globals['_STOPREPLICAREQUEST']._serialized_end=1475
  _globals['_STOPREPLICARESPONSE']._serialized_start=1477
  _globals['_STOPREPLICARESPONSE']._serialized_end=1560
  _globals['_RESTARTQUERYREQUEST']._serialized_start=1562
  _globals['_RESTARTQUERYREQUEST']._serialized_end=1616
  _globals['_RESTARTQUERYRESPONSE']._serialized_start=1619
  _globals['_RESTARTQUERYRESPONSE']._serialized_end=1811
  _globals['_RESTARTQUERYRESPONSE_ERRORSERIALSENTRY']._serialized_start=1760
  _globals['_RESTARTQUERYRESPONSE_ERRORSERIALSENTRY']._serialized_end=1811
  _globals['_STOPQUERYREQUEST']._serialized_start=1813
  _globals['_STOPQUERYREQUEST']._serialized_end=1864
  _globals['_STOPQUERYRESPONSE']._serialized_start=1867
  _globals['_STOPQUERYRESPONSE']._serialized_end=2053
  _globals['_STOPQUERYRESPONSE_ERRORSERIALSENTRY']._serialized_start=1760
  _globals['_STOPQUERYRESPONSE_ERRORSERIALSENTRY']._serialized_end=1811
  _globals['_DETERMINEDISPATCHERREQUEST']._serialized_start=2055
  _globals['_DETERMINEDISPATCHERREQUEST']._serialized_end=2164
  _globals['_DETERMINEDISPATCHERRESPONSE']._serialized_start=2166
  _globals['_DETERMINEDISPATCHERRESPONSE']._serialized_end=2273
  _globals['_PQDBSERVERCONFIGMESSAGE']._serialized_start=2276
  _globals['_PQDBSERVERCONFIGMESSAGE']._serialized_end=2587
  _globals['_ASSIGNMENTPOLICYCONFIGMESSAGE']._serialized_start=2590
  _globals['_ASSIGNMENTPOLICYCONFIGMESSAGE']._serialized_end=2725
  _globals['_GETCONFIGURATIONREQUEST']._serialized_start=2727
  _globals['_GETCONFIGURATIONREQUEST']._serialized_end=2768
  _globals['_GETCONFIGURATIONRESPONSE']._serialized_start=2770
  _globals['_GETCONFIGURATIONRESPONSE']._serialized_end=2875
  _globals['_CONTROLLERCONFIGURATIONMESSAGE']._serialized_start=2878
  _globals['_CONTROLLERCONFIGURATIONMESSAGE']._serialized_end=3973
  _globals['_ROUTINGCONFIGMESSAGE']._serialized_start=3975
  _globals['_ROUTINGCONFIGMESSAGE']._serialized_end=4046
  _globals['_CONTROLLERTEMPQUEUEDETAILSMESSAGE']._serialized_start=4048
  _globals['_CONTROLLERTEMPQUEUEDETAILSMESSAGE']._serialized_end=4146
  _globals['_CONSOLECONFIGURATIONTYPEMESSAGE']._serialized_start=4149
  _globals['_CONSOLECONFIGURATIONTYPEMESSAGE']._serialized_end=5050
  _globals['_CONSOLECONFIGURATIONTYPEMESSAGE_PROPERTIESENTRY']._serialized_start=4970
  _globals['_CONSOLECONFIGURATIONTYPEMESSAGE_PROPERTIESENTRY']._serialized_end=5019
  _globals['_EXTRACOLUMNMESSAGE']._serialized_start=5052
  _globals['_EXTRACOLUMNMESSAGE']._serialized_end=5108
  _globals['_SERVERSELECTIONSTATUSREQUEST']._serialized_start=5110
  _globals['_SERVERSELECTIONSTATUSREQUEST']._serialized_end=5156
  _globals['_SERVERSELECTIONSTATUSRESPONSE']._serialized_start=5158
  _globals['_SERVERSELECTIONSTATUSRESPONSE']._serialized_end=5205
  _globals['_SERVERSELECTIONADMINREQUEST']._serialized_start=5207
  _globals['_SERVERSELECTIONADMINREQUEST']._serialized_end=5269
  _globals['_SERVERSELECTIONADMINRESPONSE']._serialized_start=5271
  _globals['_SERVERSELECTIONADMINRESPONSE']._serialized_end=5317
  _globals['_GETSCRIPTPATHSREQUEST']._serialized_start=5320
  _globals['_GETSCRIPTPATHSREQUEST']._serialized_end=5601
  _globals['_GETSCRIPTPATHSRESPONSE']._serialized_start=5603
  _globals['_GETSCRIPTPATHSRESPONSE']._serialized_end=5644
  _globals['_GETSCRIPTREQUEST']._serialized_start=5647
  _globals['_GETSCRIPTREQUEST']._serialized_end=5943
  _globals['_GETSCRIPTRESPONSE']._serialized_start=5945
  _globals['_GETSCRIPTRESPONSE']._serialized_end=6004
  _globals['_STATUSUPDATEREQUEST']._serialized_start=6007
  _globals['_STATUSUPDATEREQUEST']._serialized_end=6382
  _globals['_STATUSUPDATERESPONSE']._serialized_start=6384
  _globals['_STATUSUPDATERESPONSE']._serialized_end=6406
  _globals['_RELOADCONFIGREQUEST']._serialized_start=6408
  _globals['_RELOADCONFIGREQUEST']._serialized_end=6445
  _globals['_RELOADCONFIGRESPONSE']._serialized_start=6447
  _globals['_RELOADCONFIGRESPONSE']._serialized_end=6469
  _globals['_SUBSCRIBEREQUEST']._serialized_start=6471
  _globals['_SUBSCRIBEREQUEST']._serialized_end=6505
  _globals['_SUBSCRIBERESPONSE']._serialized_start=6508
  _globals['_SUBSCRIBERESPONSE']._serialized_end=6784
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

import base64
import logging
from json import loads, dumps
import io
import time
import urllib.request
import urllib.parse
from datetime import datetime
from typing import Callable, Dict, Optional, List, Union

import grpc
import pydeephaven
from pydeephaven._table_ops import FetchTableOp
from pydeephaven.table import Table
from deephaven_enterprise.proto.persistent_query_pb2 import PersistentQueryConfigMessage, PersistentQueryInfoMessage
import deephaven_enterprise.client
from pydeephaven.dherror import DHError
from pydeephaven.ticket import Ticket

from deephaven_enterprise.client.auth import AuthClient
from deephaven_enterprise.client.controller import ControllerClient

from pydeephaven.session import SharedTicket
import importlib.util
import atexit

class SessionManager:
    """
    The SessionManager authenticates to the Deephaven Enterprise server and allows you to create sessions for DnD
    workers by either creating a new temporary Persistent Query or connecting to an existing Persistent Query.
    """
    config: Dict[str, str]
    json_connection_info: str
    auth_client: AuthClient
    controller_client: ControllerClient
    active_sessions: List["DndSession"]
    log: logging.Logger
    client_name: str
    owned_channels: List[grpc.Channel]

    def __init__(self, url: str = None, json: str = None, client_name: str = None, rpc_timeout_seconds: int = 120):
        """
        Create a SessionManager for the specified JSON, which may either be a string or a URL to download.
        The Deephaven server typically provides the JSON as "https://host:port/iris/connection.json".  Exactly one
        of url or json must be provided.

        The JSON must have the following parameters: auth_host, auth_port, controller_host, controller_port.  If the
        truststore_url is set, then a trust store PEM file is downloaded from the given URL.  If the
        override_authorities parameter is set then the authority used for the TLS connection to the authentication server
        is the value of the auth_authority parameter (or "authserver" if unspecified) and the TLS connection to the
        controller uses the value of the controller_authority parameter (or for backwards compatibility "authserver"
        if unspecified).

        :param url: a URL to get the JSON connection information from
        :param json: a JSON document containing connection information
        :client_name: a descriptive string identifying this client that will be used in the server for logging
        :rpc_timeout_seconds: the timeout period in seconds for RPCs. Defaults to 120 if not provided.
        """
        self.log = logging.getLogger("deephaven_enterprise.session_manager")
        in_worker: bool = False
        if url is not None:
            if json is not None:
                raise Exception("url and json are mutually exclusive")
            urllib.parse.urlparse(url)
            json = urllib.request.urlopen(url).read()
        elif json is None:
            try:
                import deephaven_enterprise.client_worker_auth
                # If we've gotten here, this means we are local to a worker so should generate our own connection.json and then authenticate with a delegate token
                json = deephaven_enterprise.client_worker_auth.get_connection_json()
                in_worker = True
            except ImportError:
                raise Exception("Must specify url or json")

        self.client_name = client_name
        self.config = loads(json)
        self.active_sessions = []
        self.owned_channels = []

        truststore_url: Optional[str] = self.config.get("truststore_url")
        if truststore_url is not None and truststore_url != "":
            self.truststore_pem = urllib.request.urlopen(truststore_url).read()
        else:
            self.truststore_pem = None

        self.auth_client = self.create_auth_client()
        self.controller_client = self.create_controller_client()

        if in_worker:
            deephaven_enterprise.client_worker_auth.delegate_authentication(
                self.auth_client)
            self.__init_controller()

        atexit.register(self.close)

    def create_auth_client(self, auth_host: str = None, rpc_timeout_seconds: int = 120) -> AuthClient:
        """
        Create the authentication client for this session manager.

        :param auth_host: the host to connect to, defaults to the first host in the JSON config's auth_host list
        :param rpc_timeout_seconds: the timeout period in seconds for RPCs. Defaults to 120 if not provided.
        :return: the authentication client
        """
        auth_port = self.config.get("auth_port")
        auth_host = self.config.get("auth_host")[0] if auth_host is None else auth_host

        auth_channel_options = []

        if self.config.get("authentication_service_config"):
            auth_channel_options.append(("grpc.enable_retries", 1))
            auth_channel_options.append(("grpc.service_config", SessionManager._clamp_retries(
                self.config.get("authentication_service_config"), self.log)))

        if self.truststore_pem is None:
            return AuthClient(
                auth_host, auth_port,
                channel_options=auth_channel_options, client_name=self.client_name)
        else:
            credentials = grpc.ssl_channel_credentials(
                root_certificates=self.truststore_pem)

            auth_target = auth_host + ":" + str(auth_port)
            if self.config.get("override_authorities"):
                authority: str = self.config.get("auth_authority") if self.config.get(
                    "auth_authority") else "authserver"
                auth_channel_options.append(
                    ('grpc.ssl_target_name_override', authority))

            auth_channel: grpc.Channel = grpc.secure_channel(
                auth_target, credentials, auth_channel_options)
            self.owned_channels.append(auth_channel)
            return AuthClient(channel=auth_channel, rpc_timeout_seconds=rpc_timeout_seconds)

    def create_controller_client(self, rpc_timeout_seconds: int = 120) -> ControllerClient:
        """
        Create the controller client for this session manager.

        :param rpc_timeout_seconds: the timeout period in seconds for RPCs. Defaults to 120 if not provided.
        :return: the controller client
        """
        controller_port = self.config.get("controller_port")
        controller_host = self.config.get("controller_host")

        controller_channel_options = []

        if self.config.get("controller_service_config"):
            controller_channel_options.append(("grpc.enable_retries", 1))
            controller_channel_options.append(("grpc.service_config", SessionManager._clamp_retries(
                self.config.get("controller_service_config"), self.log)))

        if self.truststore_pem is None:
            return ControllerClient(
                controller_host, controller_port,
                channel_options=controller_channel_options,
                client_name=self.client_name)
        else:
            credentials = grpc.ssl_channel_credentials(
                root_certificates=self.truststore_pem)

            if self.config.get("override_authorities"):
                authority = self.config.get("controller_authority") if self.config.get(
                    "controller_authority") else "authserver"
                controller_channel_options.append(
                    ('grpc.ssl_target_name_override', authority))

            controller_target = controller_host + ":" + str(controller_port)
            controller_channel: grpc.Channel = grpc.secure_channel(controller_target, credentials,
                                                                   controller_channel_options)
            self.owned_channels.append(controller_channel)
            return ControllerClient(
                channel=controller_channel,
                client_name=self.client_name,
                rpc_timeout_seconds=rpc_timeout_seconds)

    @staticmethod
    def _clamp_retries(config_json: str, log: logging.Logger, num: int = 5) -> str:
        """
        If we use the default Java configuration of 60 retries, then we end up with ugly warning messages emitted by gRPC of:
         E0724 09:20:06.453121678 1253749 retry_service_config.cc:145]          service config: clamped retryPolicy.maxAttempts at 5

        :param config_json: the configuration json to mutate
        :param num: the maximum number of retries permitted
        """
        try:
            modified = False
            as_dict = loads(config_json)
            methodConfig = as_dict["methodConfig"]
            for method in methodConfig:
                retryPolicy = method["retryPolicy"]
                maxAttempts = retryPolicy["maxAttempts"]
                if maxAttempts > 5:
                    retryPolicy["maxAttempts"] = num
                    modified = True
                    log.debug(
                        "Clamping maximum gRPC retries to %d." % num)

            if modified:
                return dumps(as_dict)
        except Exception as e:
            print(e)
        return config_json

    def external_login(self, auth_key: Union[str, bytes]) -> None:
        """
        Authenticate to the server using an external authentication method, such as SAML or a custom, non-standard external
        authentication module.

        This requires that the server be configured with the external authentication module and that the external
        authentication module should be configured to accept the provided auth_key.

        :param auth_key: the authentication key used for external login.  This must be a string or bytes.
        """
        self.auth_client.external_login(auth_key)
        self.__init_controller()

    def password(self, user: str, password: str, effective_user: str = None):
        """
        Authenticates to the server using a username and password.

        :param user: the user to authenticate
        :param password: the user's password
        :param effective_user: the user to operate as, defaults to the user to authenticate
        """
        self.auth_client.password(user, password, effective_user)
        self.__init_controller()

    def saml(self):
        """
        Authenticate using SAML, which must be configured on the server.
        """
        if not "saml_sso_uri" in self.config:
            raise Exception("SAML URI is not defined in connection.json")
        self.auth_client.saml(self.config["saml_sso_uri"])
        self.__init_controller()

    def upload_key(self, pubtext: str):
        """
        Upload the provided public key to the Deephaven server.
        """
        if not "acl_write_server" in self.config:
            raise Exception("ACL Write URI is not defined in connection.json")

        self.auth_client.upload_key(pubtext, self.config["acl_write_server"])

    def delete_key(self, pubtext: str):
        """
        Delete the specified public key from the Deephaven server.
        """
        if not "acl_write_server" in self.config:
            raise Exception("ACL Write URI is not defined in connection.json")

        self.auth_client.upload_key(
            pubtext, self.config["acl_write_server"], delete=True)

    def private_key(self, file: Union[str, io.StringIO]):
        """
        Authenticate to the server using a Deephaven format private key file.

        https://deephaven.io/enterprise/docs/resources/how-to/connect-from-java/#instructions-for-setting-up-private-keys

        :param file: a string file name containing the private key produced by generate-iris-keys, or alternatively an
        io.StringIO instance (which may be closed after it is read)
        """
        self.auth_client.private_key(file)
        self.__init_controller()

    def close(self):
        """
        Terminate this session managers connection to the authentication server and controller.
        """
        # We should close the queries in order to delete the sessions:
        while len(self.active_sessions) > 0:
            session = self.active_sessions.pop()
            session.close()
        self.auth_client.close()
        self.controller_client.close()
        for channel in self.owned_channels:
            try:
                channel.close()
            except grpc.RpcError as e:
                self.log.warning("Failed to close channel: %s", e)

    def ping(self):
        """
        Send a ping to the authentication server and controller.
        :return: False if either ping was not sent, True if both pings were sent.
        """
        if not self.auth_client.ping():
            return False
        return self.controller_client.ping()

    def connect_to_new_worker(self, name, heap_size_gb: float,
                              server: str = None,
                              extra_jvm_args: List[str] = None,
                              extra_environment_vars: List[str] = None,
                              engine: str = "DeephavenCommunity",
                              auto_delete_timeout: Optional[int] = 600,
                              admin_groups: List[str] = None,
                              viewer_groups: List[str] = None,
                              timeout_seconds: float = 60,
                              configuration_transformer: Callable[[
                                  PersistentQueryConfigMessage], PersistentQueryConfigMessage] = None,
                              session_arguments: Dict[str, any] = None) -> "DndSession":
        """
        Create a new worker (as a temporary PersistentQuery) and establish a session to it.

        :param name: the name of the persistent query.  Defaults to None, which means a name based on the current time is used
        :param heap_size_gb: the heap size of the worker
        :param server: the server to connect to. Defaults to None, which means the first available server
        :param extra_jvm_args: extra JVM arguments for starting the worker. Defaults to None.
        :param extra_environment_vars: extra Environment variables for the worker. Defaults to None.
        :param engine: which engine (worker kind) to use for the backend worker. Defaults to None, which means
               "DeephavenCommunity"
        :param auto_delete_timeout: after how many seconds should the query be automatically deleted after inactivity.
                Defaults to ten minutes.  If none, auto-delete is disabled.  If zero, the query is deleted immediately after a
                client connection is lost
        :param admin_groups: list of groups that may administer the query.  Defaults to None, which means only the
               current user may administer the query.
        :param viewer_groups: list of groups that may view the query.  Defaults to None, which means only the current
               user may view the query.
        :param timeout_seconds: how long to wait for the query to start.  Defaults to 60 seconds.
        :param configuration_transformer: a function that can replace (or edit) the automatically generated persistent
               query configuration, enabling you to set more advanced options than the other function parameters provide.  Defaults to None.
        :param session_arguments: a dictionary of additional arguments to pass to the pydeephaven.Session created and wrapped by a DndSession
        :return: a session connected to a new Interactive Console PQ worker
        """
        now: datetime = datetime.now()
        start_time = time.time()

        # Create the Configuration
        name = "Python Console " + str(now) if name is None else name
        temp_config: PersistentQueryConfigMessage
        temp_config = self.controller_client.make_temporary_config(name, heap_size_gb,
                                                                   server=server,
                                                                   extra_jvm_args=extra_jvm_args,
                                                                   extra_environment_vars=extra_environment_vars,
                                                                   engine=engine,
                                                                   auto_delete_timeout=auto_delete_timeout,
                                                                   admin_groups=admin_groups,
                                                                   viewer_groups=viewer_groups)
        if configuration_transformer is not None:
            temp_config = configuration_transformer(temp_config)
        serial: int = self.controller_client.add_query(temp_config)
        self.controller_client.restart_query(serial)

        print("Waiting for query \"%s\" to be ready" % name)
        timeout_left = timeout_seconds - (time.time() - start_time)
        pqinfo: PersistentQueryInfoMessage = self.__wait_for_ready(
            serial, timeout_left)

        if pqinfo is None:
            time_elapsed = time.time() - start_time
            self.controller_client.delete_query(serial)
            raise Exception("Persistent Query did not start after %.1f seconds, serial=%s, name=%s" % (
                time_elapsed, serial, name))

        if self.controller_client.is_terminal(pqinfo.state.status):
            self.controller_client.delete_query(serial)
            raise Exception(
                "Query is in terminal state.  Exception details: ", pqinfo.state.exceptionDetails)

        if not self.controller_client.is_running(pqinfo.state.status):
            time_elapsed = time.time() - start_time
            self.controller_client.delete_query(serial)
            status_name = self.controller_client.status_name(
                pqinfo.state.status)
            details = "" if pqinfo.state.statusDetails is None else pqinfo.state.statusDetails
            raise Exception("Query '%s' is not running after %.1f seconds.  Status: %s %s" % (
                name, time_elapsed, status_name, details))

        print("Connecting to new query \"%s\", ProcessInfoId=\"%s\"" %
              (pqinfo.config.name, pqinfo.state.connectionDetails.processInfoId))

        return self.__make_session(pqinfo, serial, auto_delete=True, session_arguments=session_arguments)

    def __wait_for_ready(self, serial, timeout_seconds) -> Optional[deephaven_enterprise.proto.persistent_query_pb2.PersistentQueryInfoMessage]:
        last_status = None
        last_status_details = None
        pqinfo: Optional[deephaven_enterprise.proto.persistent_query_pb2.PersistentQueryInfoMessage] = None
        deadline = time.time() + timeout_seconds
        while deadline > time.time():
            pq_info_map, map_version = self.controller_client.map_and_version()
            pqinfo = pq_info_map.get(serial, None)
            if pqinfo:
                if not pqinfo.HasField("state"):
                    if last_status is not None:
                        print(datetime.now(), "No state for ", pqinfo.config.name)
                        last_status = None
                        last_status_details = None
                else:
                    status = pqinfo.state.status
                    status_details = pqinfo.state.statusDetails
                    if self.controller_client.is_running(status):
                        break
                    elif self.controller_client.is_terminal(status):
                        break
                    if status != last_status or status_details != last_status_details:
                        status_name = self.controller_client.status_name(status)
                        print(datetime.now(), pqinfo.config.name,
                              ": Query Status: ", status_name, status_details)
                    last_status = status
                    last_status_details = status_details
            # Only to continue if there are actual changes to the pq info map
            self.controller_client.wait_for_change_from_version(map_version=map_version, timeout_seconds=deadline - time.time())
        return pqinfo

    def connect_to_persistent_query(self, name: str = None, serial: int = None, session_arguments: Dict[str, any] = None) -> "DndSession":
        """
        Connect to an existing persistent query by name or serial number.  The query must be running.

        :param name: the name of the persistent query to connect to
        :param serial: the serial number of the persistent query to connect to
        :param session_arguments: a dictionary of additional arguments to pass to the pydeephaven.Session created and wrapped by a DndSession

        :return: a session connected to the persistent query
        """
        pqinfo: Optional[PersistentQueryInfoMessage]

        if serial is not None:
            pqinfo = self.controller_client.get(serial)
            if pqinfo is None:
                raise Exception(
                    "Could not find query with serial: " + str(serial))
            if name is not None:
                if pqinfo.config.name != name:
                    raise Exception(
                        "Name (" + name + ") and serial number (" + str(serial) + ") are inconsistent: " + str(
                            pqinfo.config))
        elif name is not None:
            pqinfo = None
            for search_pq in self.controller_client.map().values():
                if search_pq.config.name == name:
                    pqinfo = search_pq
                    serial = search_pq.config.serial
                    break
            if pqinfo is None:
                raise Exception("Could not find query with name: " + name)
        else:
            raise Exception(
                "You must specify either name or serial to connect_to_persistent_query.")

        if self.controller_client.is_terminal(pqinfo.state.status):
            raise Exception(
                "Query is in terminal state.  Exception details: " + str(pqinfo.state.exceptionDetails))

        if not self.controller_client.is_running(pqinfo.state.status):
            status_name = self.controller_client.status_name(
                pqinfo.state.status)
            raise Exception("Query '%s' is not running, state is %s" %
                            (name, status_name))

        return self.__make_session(pqinfo, serial, auto_delete=False, session_arguments=session_arguments)

    def __make_session(self, pqinfo, serial, auto_delete: bool, session_arguments: Dict[str, any] = None):
        url: str = pqinfo.state.connectionDetails.grpcUrl
        if url is None or url == "":
            if pqinfo.state.engineVersion == "":
                raise Exception("Not a Community engine: " +
                                pqinfo.config.name + "(" + str(pqinfo.config.serial) + ")")
            raise Exception("gRPC is not available for " +
                            pqinfo.config.name + "(" + str(pqinfo.config.serial) + ")")
        parsed = urllib.parse.urlparse(url)

        envoy_prefix: str = pqinfo.state.connectionDetails.envoyPrefix
        extra_headers = {b'envoy-prefix': bytes(envoy_prefix, 'us-ascii')}

        token: deephaven_enterprise.proto.auth_pb2.Token = self.auth_client.get_token(
            "RemoteQueryProcessor")
        token_base64 = base64.b64encode(
            token.SerializeToString()).decode('us-ascii')

        session = DndSession(session_manager=self, pqinfo=pqinfo, host=parsed.hostname, port=parsed.port, auth_token=token_base64,
                             extra_headers=extra_headers, delete_on_close=serial if auto_delete else None,
                             session_type=pqinfo.config.scriptLanguage,
                             session_arguments=session_arguments
                             )

        return session

    def __init_controller(self):
        self.controller_client.set_auth_client(self.auth_client)
        self.controller_client.subscribe()


class DndSession(pydeephaven.session.Session):
    """
    Wrapper around a basic Community session.  For queries that are ephemeral, they are explicitly deleted on session
    close.
    """
    delete_on_close: Optional[int]
    session_manager: SessionManager
    __pqinfo: PersistentQueryInfoMessage
    __barrage_session: barrage.BarrageSession
    __barrage_scope: LivenessScope

    def __init__(self, session_manager: SessionManager,
                 pqinfo: PersistentQueryInfoMessage,
                 host: str = None, port: int = None,
                 auth_type: str = "io.deephaven.proto.auth.Token", auth_token: str = None,
                 extra_headers: Dict[bytes, bytes] = None, delete_on_close: int = None,
                 session_type: str = 'python',
                 session_arguments: Dict[str, any] = None):
        # We initialize enough to delete the query
        self.delete_on_close = delete_on_close
        self.session_manager = session_manager
        self.session_manager.active_sessions.append(self)

        # Then call the super, the base class __del__ method is still invoked by the Python runtime.  The
        # pydeephaven.Session __del__ method calls close() - which we override.  If we call into the super close,
        # in those cases, the uninitialized Session object will throw from the close method.
        try:
            super().__init__(host=host, port=port, auth_type=auth_type, auth_token=auth_token, use_tls=True,
                             extra_headers=extra_headers, tls_root_certs=session_manager.truststore_pem,
                             session_type=session_type,
                             **(session_arguments if session_arguments is not None else dict()))
        except Exception as e:
            self.init_complete = False
            raise e
        # init was complete, we can close the object when it comes time
        self.init_complete = True
        self.__pqinfo = pqinfo
        self.__barrage_session = None
        self.__barrage_scope = None

    def pqinfo(self) -> PersistentQueryInfoMessage:
        """
        Retrieve the persistent query information for this query.

        :return:
        """
        return self.__pqinfo

    def close(self):
        if self.__barrage_scope is not None:
            self.__barrage_scope.release()
        if self.__barrage_session is not None:
            self.__barrage_session.close()
        if self.init_complete:
            super().close()
        if self.delete_on_close is not None:
            self.session_manager.controller_client.delete_query(
                self.delete_on_close)
            self.delete_on_close = None
        try:
            self.session_manager.active_sessions.remove(self)
        except ValueError:
            pass

    def historical_table(self, namespace: str, table_name: str):
        """
        Fetches a historical table from the database on the server.

        :param namespace: the namespace of the table
        :param table_name: the name of the table

        :return: a Table object
        :raise: DHError
        """
        return self.__fetch_table_(DatabaseTicket.historical_ticket(namespace, table_name))

    def live_table(self, namespace: str, table_name: str):
        """
        Fetches a live table from the database on the server.

        :param namespace: the namespace of the table
        :param table_name: the name of the table

        :return: a Table object
        :raise: DHError
        """
        return self.__fetch_table_(DatabaseTicket.live_ticket(namespace, table_name))

    def catalog_table(self):
        """
        Fetches the catalog table from the database on the server.

        :return: a Table object
        :raise: DHError
        """
        return self.__fetch_table_(CatalogTicket.catalog_ticket())

    def __fetch_table_(self, ticket):
        with self._r_lock:
            faketable = Table(session=self, ticket=ticket)
            try:
                table_op = FetchTableOp()
                return self.table_service.grpc_table_op(faketable, table_op)
            except Exception as e:
                if isinstance(e.__cause__, grpc.RpcError):
                    if e.__cause__.code() == grpc.StatusCode.INVALID_ARGUMENT:
                        raise DHError(
                            f"no table by the name {ticket}") from None
                raise e
            finally:
                # Explicitly close the table without releasing it (because it isn't ours)
                faketable.ticket = None
                faketable.schema = None

    def _maybe_init_barrage_session(self):
        if importlib.util.find_spec("deephaven") is None:
            raise Exception(
                "Barrage functionality is only supported in workers.")
        else:
            from deephaven import barrage
            from deephaven.liveness_scope import LivenessScope

        if self.__barrage_session is None:
            url = urllib.parse.urlparse(
                self.__pqinfo.state.connectionDetails.grpcUrl)
            host = url.hostname
            port = url.port

            auth_type = "io.deephaven.proto.auth.Token"
            token = self.session_manager.auth_client.get_token(
                "RemoteQueryProcessor")
            auth_token = base64.b64encode(
                token.SerializeToString()).decode('us-ascii')

            envoy_prefix = self.__pqinfo.state.connectionDetails.envoyPrefix
            extra_headers = None if envoy_prefix is None else {
                "envoy-prefix": envoy_prefix}

            self.__barrage_session = barrage.barrage_session(host=host, port=port, auth_type=auth_type, auth_token=auth_token,
                                                             use_tls=True, tls_root_certs=self.session_manager.truststore_pem, extra_headers=extra_headers)
            self.__barrage_scope = LivenessScope()

        return self.__barrage_session

    def barrage_subscribe(self, table_ref: Table) -> deephaven.table.Table:
        """
        Creates a local table of the specified table reference that subscribes to updates.

        :param table_ref: table reference to create local table of
        :return: local table subscribed to updates
        """
        barrage_session = self._maybe_init_barrage_session()

        ticket = SharedTicket.random_ticket()
        self.publish_table(ticket, table_ref)

        with self.__barrage_scope.open():
            barrage_table = barrage_session.subscribe(ticket.bytes)

        return barrage_table

    def barrage_snapshot(self, table_ref: Table) -> deephaven.table.Table:
        """
        Creates a local table snapshot of the specified table reference.

        :param table_ref: table reference to create local table of
        :return: local table snapshot
        """
        barrage_session = self._maybe_init_barrage_session()

        ticket = SharedTicket.random_ticket()
        self.publish_table(ticket, table_ref)
        return barrage_session.snapshot(ticket.bytes)

    def barrage_session(self) -> barrage.BarrageSession:
        """
        Returns a Barrage session connected to the same Persistent Query as this DndSession.

        :return: the Barrage session connected to the same Persistent Query as this DndSession
        """

        return self._maybe_init_barrage_session()

class DatabaseTicket(Ticket):
    """A DatabaseTicket is ticket that references a table by namespace and table name"""
    def __init__(self, ticket_bytes: bytes):
        """Initializes a DatabaseTicket.

        Args:
            ticket_bytes (bytes): the raw bytes for the ticket
        """
        if not ticket_bytes:
            raise DHError('DatabaseTicket: ticket is None')
        elif not ticket_bytes.startswith(b'd/'):
            raise DHError(f'DatabaseTicket: ticket {ticket_bytes} is not a database table ticket')
        elif len(ticket_bytes.split(b'/')) != 4:
            raise DHError(f'DatabaseTicket: ticket {ticket_bytes} is not in the correct format')

        splits = ticket_bytes.split(b'/')

        self.type = splits[1].decode(encoding='ascii')
        self.namespace = splits[2].decode(encoding='ascii')
        self.table_name = splits[3].decode(encoding='ascii')

        super().__init__(ticket_bytes)

    @classmethod
    def live_ticket(cls, namespace: str, table_name: str) -> DatabaseTicket:
        """Creates a ticket that references a database intraday table

        Args:
            namespace (str): the namespace
            table_name (str): the table name

        Returns:
            a DatabaseTicket
        """
        return DatabaseTicket._db_ticket('live', namespace, table_name)

    @classmethod
    def historical_ticket(cls, namespace: str, table_name: str) -> DatabaseTicket:
        """Creates a ticket that references a database historical table

        Args:
            namespace (str): the namespace
            table_name (str): the table name

        Returns:
            a DatabaseTicket
        """
        return DatabaseTicket._db_ticket('hist', namespace, table_name)

    @classmethod
    def _db_ticket(cls, type: str, namespace: str, table_name: str) -> DatabaseTicket:
        """Creates a ticket that references a database table

        Args:
            type (str): the type (live, hist, or catalog)
            namespace (str): the namespace
            table_name (str): the table name

        Returns:
            an LiveTicket
        """
        if not type or type not in ['live', 'hist', 'catalog']:
            raise DHError('db_ticket: type must be one of [live, hist, catalog]')
        if not namespace:
            raise DHError('db_ticket: namespace must be a non-empty string')
        if not table_name:
            raise DHError('db_ticket: table_name must be a non-empty string')

        return cls(ticket_bytes=f'd/{type}/{namespace}/{table_name}'.encode(encoding='ascii'))

class CatalogTicket(Ticket):
    """A CatalogTicket is ticket that the catalog table"""
    def __init__(self):

        super().__init__(f'd/catalog'.encode(encoding='ascii'))

    @classmethod
    def catalog_ticket(cls):
        """Creates a ticket that references the database catalog table

        Returns:
            a CatalogTicket
        """
        return cls()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from deephaven_enterprise.proto import auth_pb2 as deephaven__enterprise_dot_proto_dot_auth__pb2
from deephaven_enterprise.proto import common_pb2 as deephaven__enterprise_dot_proto_dot_common__pb2

GRPC_GENERATED_VERSION = '1.63.0'
GRPC_VERSION = grpc.__version__
EXPECTED_ERROR_RELEASE = '1.65.0'
SCHEDULED_RELEASE_DATE = 'June 25, 2024'
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    warnings.warn(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in deephaven_enterprise/proto/auth_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
        + f' This warning will become an error in {EXPECTED_ERROR_RELEASE},'
        + f' scheduled for release on {SCHEDULED_RELEASE_DATE}.',
        RuntimeWarning
    )


class AuthApiStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ping = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/ping',
                request_serializer=deephaven__enterprise_dot_proto_dot_common__pb2.PingRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_common__pb2.PingResponse.FromString,
                _registered_method=True)
        self.authenticateByPassword = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByPassword',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPasswordRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPasswordResponse.FromString,
                _registered_method=True)
        self.authenticateByPublicKey = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByPublicKey',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPublicKeyRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPublicKeyResponse.FromString,
                _registered_method=True)
        self.authenticateByDelegateToken = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByDelegateToken',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByDelegateTokenRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByDelegateTokenResponse.FromString,
                _registered_method=True)
        self.authenticateByExternal = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByExternal',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByExternalRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByExternalResponse.FromString,
                _registered_method=True)
        self.authenticateByCookie = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByCookie',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByCookieRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByCookieResponse.FromString,
                _registered_method=True)
        self.refreshCookie = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/refreshCookie',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.RefreshCookieRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.RefreshCookieResponse.FromString,
                _registered_method=True)
        self.invalidateCookie = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/invalidateCookie',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.InvalidateCookieRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.InvalidateCookieResponse.FromString,
                _registered_method=True)
        self.getNonce = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/getNonce',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetNonceRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetNonceResponse.FromString,
                _registered_method=True)
        self.getToken = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/getToken',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenResponse.FromString,
                _registered_method=True)
        self.getTokenAs = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/getTokenAs',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenAsRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenAsResponse.FromString,
                _registered_method=True)
        self.verifyToken = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/verifyToken',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyTokenRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyTokenResponse.FromString,
                _registered_method=True)
        self.reload = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/reload',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.ReloadRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.ReloadResponse.FromString,
                _registered_method=True)
        self.verifyChallenge = channel.unary_unary(
                '/io.deephaven.proto.auth.grpc.AuthApi/verifyChallenge',
                request_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyChallengeRequest.SerializeToString,
                response_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyChallengeResponse.FromString,
                _registered_method=True)


class AuthApiServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ping(self, request, context):
        """Simple ping method that allows a client to sample the latency to the server.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def authenticateByPassword(self, request, context):
        """
        Authenticate methods validate credentials and register authentication state (unless asked to verify only)


        Authenticate by password.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def authenticateByPublicKey(self, request, context):
        """Authenticate by public key challenge; should have obtained a nonce earlier via getNonce rpc.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def authenticateByDelegateToken(self, request, context):
        """Authenticate by delegate token; should have obtained a token earlier from another service that
        created it and forwarded it.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def authenticateByExternal(self, request, context):
        """Authenticate by an external method (eg, active directory).
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def authenticateByCookie(self, request, context):
        """Authenticate by providing an already existing cookie and the user context that cookie should be associated with.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def refreshCookie(self, request, context):
        """Refresh a cookie to maintain credentials.  Clients are expected to try to refresh cookies
        with enough time in advance before expiration; a reasonable default is half way through from the time
        the cookie was obtained with its deadline, and the actual deadline time.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def invalidateCookie(self, request, context):
        """Clients that are about to terminate are expected to invalidate their credentials before going away.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getNonce(self, request, context):
        """Get a nonce for public key authentication.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getToken(self, request, context):
        """Get a token for a three-way handshake.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getTokenAs(self, request, context):
        """Get a token for a three-way handshake as a particular user.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def verifyToken(self, request, context):
        """Verify a token provided by another service.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def reload(self, request, context):
        """Request this server to reload its configuration.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def verifyChallenge(self, request, context):
        """Verify a nonce challenge response for another server (not used directly by clients, only server-server)
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AuthApiServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ping': grpc.unary_unary_rpc_method_handler(
                    servicer.ping,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_common__pb2.PingRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_common__pb2.PingResponse.SerializeToString,
            ),
            'authenticateByPassword': grpc.unary_unary_rpc_method_handler(
                    servicer.authenticateByPassword,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPasswordRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPasswordResponse.SerializeToString,
            ),
            'authenticateByPublicKey': grpc.unary_unary_rpc_method_handler(
                    servicer.authenticateByPublicKey,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPublicKeyRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPublicKeyResponse.SerializeToString,
            ),
            'authenticateByDelegateToken': grpc.unary_unary_rpc_method_handler(
                    servicer.authenticateByDelegateToken,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByDelegateTokenRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByDelegateTokenResponse.SerializeToString,
            ),
            'authenticateByExternal': grpc.unary_unary_rpc_method_handler(
                    servicer.authenticateByExternal,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByExternalRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByExternalResponse.SerializeToString,
            ),
            'authenticateByCookie': grpc.unary_unary_rpc_method_handler(
                    servicer.authenticateByCookie,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByCookieRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByCookieResponse.SerializeToString,
            ),
            'refreshCookie': grpc.unary_unary_rpc_method_handler(
                    servicer.refreshCookie,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.RefreshCookieRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.RefreshCookieResponse.SerializeToString,
            ),
            'invalidateCookie': grpc.unary_unary_rpc_method_handler(
                    servicer.invalidateCookie,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.InvalidateCookieRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.InvalidateCookieResponse.SerializeToString,
            ),
            'getNonce': grpc.unary_unary_rpc_method_handler(
                    servicer.getNonce,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetNonceRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetNonceResponse.SerializeToString,
            ),
            'getToken': grpc.unary_unary_rpc_method_handler(
                    servicer.getToken,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenResponse.SerializeToString,
            ),
            'getTokenAs': grpc.unary_unary_rpc_method_handler(
                    servicer.getTokenAs,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenAsRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenAsResponse.SerializeToString,
            ),
            'verifyToken': grpc.unary_unary_rpc_method_handler(
                    servicer.verifyToken,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyTokenRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyTokenResponse.SerializeToString,
            ),
            'reload': grpc.unary_unary_rpc_method_handler(
                    servicer.reload,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.ReloadRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.ReloadResponse.SerializeToString,
            ),
            'verifyChallenge': grpc.unary_unary_rpc_method_handler(
                    servicer.verifyChallenge,
                    request_deserializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyChallengeRequest.FromString,
                    response_serializer=deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyChallengeResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'io.deephaven.proto.auth.grpc.AuthApi', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AuthApi(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ping(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/ping',
            deephaven__enterprise_dot_proto_dot_common__pb2.PingRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_common__pb2.PingResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def authenticateByPassword(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByPassword',
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPasswordRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPasswordResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def authenticateByPublicKey(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByPublicKey',
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPublicKeyRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByPublicKeyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def authenticateByDelegateToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByDelegateToken',
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByDelegateTokenRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByDelegateTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def authenticateByExternal(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByExternal',
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByExternalRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByExternalResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def authenticateByCookie(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/authenticateByCookie',
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByCookieRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.AuthenticateByCookieResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def refreshCookie(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/refreshCookie',
            deephaven__enterprise_dot_proto_dot_auth__pb2.RefreshCookieRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.RefreshCookieResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def invalidateCookie(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/invalidateCookie',
            deephaven__enterprise_dot_proto_dot_auth__pb2.InvalidateCookieRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.InvalidateCookieResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def getNonce(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/getNonce',
            deephaven__enterprise_dot_proto_dot_auth__pb2.GetNonceRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.GetNonceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def getToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/getToken',
            deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def getTokenAs(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/getTokenAs',
            deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenAsRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.GetTokenAsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def verifyToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/verifyToken',
            deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyTokenRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyTokenResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def reload(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/reload',
            deephaven__enterprise_dot_proto_dot_auth__pb2.ReloadRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.ReloadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def verifyChallenge(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/io.deephaven.proto.auth.grpc.AuthApi/verifyChallenge',
            deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyChallengeRequest.SerializeToString,
            deephaven__enterprise_dot_proto_dot_auth__pb2.VerifyChallengeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deephaven_enterprise/proto/auth_service.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from deephaven_enterprise.proto import common_pb2 as deephaven__enterprise_dot_proto_dot_common__pb2
from deephaven_enterprise.proto import auth_pb2 as deephaven__enterprise_dot_proto_dot_auth__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-deephaven_enterprise/proto/auth_service.proto\x12\x1cio.deephaven.proto.auth.grpc\x1a\'deephaven_enterprise/proto/common.proto\x1a%deephaven_enterprise/proto/auth.proto2\x93\r\n\x07\x41uthApi\x12Y\n\x04ping\x12&.io.deephaven.proto.common.PingRequest\x1a\'.io.deephaven.proto.common.PingResponse\"\x00\x12\x8b\x01\n\x16\x61uthenticateByPassword\x12\x36.io.deephaven.proto.auth.AuthenticateByPasswordRequest\x1a\x37.io.deephaven.proto.auth.AuthenticateByPasswordResponse\"\x00\x12\x8e\x01\n\x17\x61uthenticateByPublicKey\x12\x37.io.deephaven.proto.auth.AuthenticateByPublicKeyRequest\x1a\x38.io.deephaven.proto.auth.AuthenticateByPublicKeyResponse\"\x00\x12\x9a\x01\n\x1b\x61uthenticateByDelegateToken\x12;.io.deephaven.proto.auth.AuthenticateByDelegateTokenRequest\x1a<.io.deephaven.proto.auth.AuthenticateByDelegateTokenResponse\"\x00\x12\x8b\x01\n\x16\x61uthenticateByExternal\x12\x36.io.deephaven.proto.auth.AuthenticateByExternalRequest\x1a\x37.io.deephaven.proto.auth.AuthenticateByExternalResponse\"\x00\x12\x85\x01\n\x14\x61uthenticateByCookie\x12\x34.io.deephaven.proto.auth.AuthenticateByCookieRequest\x1a\x35.io.deephaven.proto.auth.AuthenticateByCookieResponse\"\x00\x12p\n\rrefreshCookie\x12-.io.deephaven.proto.auth.RefreshCookieRequest\x1a..io.deephaven.proto.auth.RefreshCookieResponse\"\x00\x12y\n\x10invalidateCookie\x12\x30.io.deephaven.proto.auth.InvalidateCookieRequest\x1a\x31.io.deephaven.proto.auth.InvalidateCookieResponse\"\x00\x12\x61\n\x08getNonce\x12(.io.deephaven.proto.auth.GetNonceRequest\x1a).io.deephaven.proto.auth.GetNonceResponse\"\x00\x12\x61\n\x08getToken\x12(.io.deephaven.proto.auth.GetTokenRequest\x1a).io.deephaven.proto.auth.GetTokenResponse\"\x00\x12g\n\ngetTokenAs\x12*.io.deephaven.proto.auth.GetTokenAsRequest\x1a+.io.deephaven.proto.auth.GetTokenAsResponse\"\x00\x12j\n\x0bverifyToken\x12+.io.deephaven.proto.auth.VerifyTokenRequest\x1a,.io.deephaven.proto.auth.VerifyTokenResponse\"\x00\x12[\n\x06reload\x12&.io.deephaven.proto.auth.ReloadRequest\x1a\'.io.deephaven.proto.auth.ReloadResponse\"\x00\x12v\n\x0fverifyChallenge\x12/.io.deephaven.proto.auth.VerifyChallengeRequest\x1a\x30.io.deephaven.proto.auth.VerifyChallengeResponse\"\x00\x42\x04H\x01P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deephaven_enterprise.proto.auth_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\001P\001'
  _globals['_AUTHAPI']._serialized_start=160
  _globals['_AUTHAPI']._serialized_end=1843
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: deephaven_enterprise/proto/controller_common.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2deephaven_enterprise/proto/controller_common.proto\x12\x1dio.deephaven.proto.controller\" \n\rErrorResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\"-\n\x0bPingRequest\x12\x13\n\x06\x63ookie\x18\x01 \x01(\x0cH\x00\x88\x01\x01\x42\t\n\x07_cookie\"\x0e\n\x0cPingResponse\"/\n\x0fNamedStringList\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06values\x18\x02 \x03(\tB\x04H\x01P\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'deephaven_enterprise.proto.controller_common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'H\001P\001'
  _globals['_ERRORRESPONSE']._serialized_start=85
  _globals['_ERRORRESPONSE']._serialized_end=117
  _globals['_PINGREQUEST']._serialized_start=119
  _globals['_PINGREQUEST']._serialized_end=164
  _globals['_PINGRESPONSE']._serialized_start=166
  _globals['_PINGRESPONSE']._serialized_end=180
  _globals['_NAMEDSTRINGLIST']._serialized_start=182
  _globals['_NAMEDSTRINGLIST']._serialized_end=229
# @@protoc_insertion_point(module_scope)
